package com.persagy.dmp.middleware.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.model.entity.BaseEntity;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.entity.response.DmpResult;
import com.persagy.dmp.middleware.service.IBaseService;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.middleware.client.DmpRwdClient;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.util.*;

/**
 * ADM 抽象实现类
 * @author Charlie Yu
 * @date 2021-08-16
 */
@Slf4j
@Service
public class AbstractBaseServiceImpl<T> implements IBaseService<T> {

    @Autowired
    private DmpRwdClient rwdClient;

    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 查询
     * @param dmpRequest
     * @return
     */
    @Override
    public CommonResult doQuery(InstanceUrlParam context, QueryCriteria dmpRequest, Class<T> clazz) {
        // 转换参数
        JSONObject para = null;
        try {
            String jsonStr = objectMapper.writeValueAsString(dmpRequest);
            para = JSONObject.parseObject(jsonStr);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            return null;
        }
        // 调用中台查询
        DmpResult<JSONArray> dmpResult = rwdClient.queryObject(context, para);
        JSONArray data = dmpResult.getData();
        // 设置返回值
        List<JSONObject> VOs = EntityTransferUtil.toMultiEntity(data);
        CommonResult response = CommonResult.success(VOs);
        Long total = dmpResult.getCount() == null ? null : dmpResult.getCount().longValue();
        response.setCount(total);
        return response;
    }

    /**
     * 将集合转换为Map
     * @param list 集合
     * @param keyField key属性名
     * @param valueField value属性名
     * @return
     */
    private static Map<String, List<String>> groupMapList(List<?> list, String keyField, String valueField) {
        if(CollUtil.isEmpty(list)) {
            return null;
        }
        Map<String, List<String>> groupMap = new HashMap<>();
        for(Object vo:list) {
            String key = (String) ReflectUtil.getFieldValue(vo, keyField);
            String value = (String) ReflectUtil.getFieldValue(vo, valueField);
            if(!StrUtil.isAllNotBlank(key, value)) {
                continue;
            }
            List<String> valueList = groupMap.get(key);
            if(CollUtil.isEmpty(valueList)) {
                valueList = new ArrayList<>();
                groupMap.put(key, valueList);
            }
            valueList.add(value);
        }
        return groupMap;
    }

    /**
     * 查询关系
     * @param context
     * @param idList
     * @param anno
     * @return
     */
  /*  private List<ObjectRelation> queryRelations(InstanceUrlParam context, Set<String>idList, CascadeColumn anno) {
        // 如果没有任何配置，不处理
        if(StrUtil.isBlank(anno.graphCode()) && StrUtil.isBlank(anno.relCode()) &&
                StrUtil.isBlank(anno.relValue())) {
            return null;
        }
        // 查询关系
        QueryCriteria queryRequest = new QueryCriteria();
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        queryRequest.setCriteria(criteria);
        boolean isSelectTo = anno.selectToObj();
        putString(criteria, "graphCode", anno.graphCode());
        putString(criteria, "relCode", anno.relCode());
        putString(criteria, "relValue", anno.relValue());
        ArrayNode array = criteria.putObject(isSelectTo?"objFrom":"objTo").putArray("$in");
        idList.forEach(id -> array.add(id));
        return SpringHelper.getBean(IAdmRelationService.class).queryByCondition(context, queryRequest);
    }*/

    /**
     * 添加字符串条件
     * @param node
     * @param key
     * @param value
     */
    private void putString(ObjectNode node, String key, String value) {
        if(StrUtil.isBlank(value)) {
            return;
        }
        node.put(key, value);
    }

    /**
     * 将子集写回到对象中
     * @param voMap 原始对象Map
     * @param idMap 主键映射<mainId, List<subId>>
     * @param vos 子对象
     * @param casField 子对象对应属性
     */
    private <V> void writeToVOs(Map<String, Object> voMap, Map<String, List<String>> idMap, List<V> vos, Field casField) {
        if(CollUtil.isEmpty(vos) || MapUtil.isEmpty(idMap)) {
            return;
        }
        // 转换为Map
        Map<String, V> subVOMap = CollUtil.fieldValueMap(vos, BaseEntity.PROP_ID);
        // 是否为集合
        boolean isList = List.class.isAssignableFrom(casField.getType());
        // 按对象循环
        for(String id:voMap.keySet()) {
            Object mainVO = voMap.get(id);
            List<String> subIdList = idMap.get(id);
            if(CollUtil.isEmpty(subIdList)) {
                continue;
            }
            // 如果是集合
            if(isList) {
                // 取出对应的对象集合
                Map<String, V> currVOMap = MapUtil.filter(subVOMap, subIdList.toArray(new String[0]));
                List<V> voList = CollUtil.newArrayList(currVOMap.values());
                ReflectUtil.setFieldValue(mainVO, casField, voList);
            } else {
                // 非集合取第一个即可
                String subId = subIdList.get(0);
                V vo = subVOMap.get(subId);
                ReflectUtil.setFieldValue(mainVO, casField, vo);
            }
        }
    }

    /**
     * 获取实际的Class
     * @param casField
     * @return
     */
    private Class getFieldRealClass(Field casField) {
        // 取到Field
        Class clazz = casField.getType();
        // 如果是集合，则取实际的对象
        if(List.class.isAssignableFrom(clazz)) {
            clazz = (Class) ((ParameterizedType) casField.getGenericType()).getActualTypeArguments()[0];
        }
        return clazz;
    }

    /**
     * 通过Class获取其对象类型
     * @param clazz
     * @return
     */
    private String getObjTypeByClass(Class clazz) {
        // 获取关联对象的类型
        String objectType = null;
        try {
            objectType = (String) ReflectUtil.getFieldValue(clazz.newInstance(), "objectType");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return objectType;
    }

    /**
     * 新增
     * @param context
     * @param clazz
     * @param voList
     * @return
     */
    @Override
    public List<T> doInsert(InstanceUrlParam context, Class<T> clazz, List<T> voList) {
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
        ArrayNode vos = EntityTransferUtil.toDmpMultiEntity(voList);
        // 调用中台新增
        DmpResult<JSONArray> response = rwdClient.createObject(context, JSONArray.parseArray(vos.toString()));
        if(!DmpResult.SUCCESS.equals(response.getResult())) {
            throw new RuntimeException("调用中台查询接口出错：" + response.getMessage());
        }
        // 转换为结果
        return EntityTransferUtil.toAdmMultiEntity(response.getData(), null, clazz);
    }

    /**
     * 修改
     * @param context
     * @param clazz
     * @param voList
     * @return
     */
    @Override
    public List<T> doUpdate(InstanceUrlParam context, Class<T> clazz, List<T> voList) {
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
        ArrayNode vos = EntityTransferUtil.toDmpMultiEntity(voList);
        // 调用中台修改
        DmpResult<cn.hutool.json.JSONArray> response = rwdClient.updateObject(context, JSONArray.parseArray(vos.toString()));
        if(!DmpResult.SUCCESS.equals(response.getResult())) {
            throw new RuntimeException("调用中台查询接口出错：" + response.getMessage());
        }
        // 转换为结果
        return EntityTransferUtil.toAdmMultiEntity(response.getData(),null, clazz);
    }

    /**
     * 删除
     * @param context
     * @param voList
     * @return
     */
    @Override
    public void doDelete(InstanceUrlParam context, List<T> voList) {
        if(CollUtil.isEmpty(voList)) {
            return;
        }
        List<String> idList = CollUtil.getFieldValues(voList, "id", String.class);
        // 调用中台修改
        DmpResult response = rwdClient.deleteObject(context, idList);
        if(!DmpResult.SUCCESS.equals(response.getResult())) {
            throw new RuntimeException("调用中台查询接口出错：" + response.getMessage());
        }
    }
}
