package com.persagy.dmp.middleware.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.basic.utils.JsonNodeUtils;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * ADM通用查询条件
 * @author Charlie Yu
 * @date 2021-08-16
 */
public class EntityTransferUtil {

    /**
     * 转换为数据中台对象
     * @param objList
     * @return
     */
    public static ArrayNode toDmpMultiEntity(List objList) {
        return JsonNodeUtils.toArrayNode(objList, "infos", null);
    }

    /**
     * 转换为数据中台对象
     * @param obj
     * @return
     */
    public static JsonNode toDmpSingleEntity(Object obj) {
        return JsonNodeUtils.toObjectNode(obj, "infos", null);
    }

    /**
     * 转换为数据中心对象
     * @param data
     * @return
     */
    public static List<JSONObject> toMultiEntity(JSONArray data) {
        List<JSONObject> result = new ArrayList<>();
        if(CollUtil.isEmpty(data)){
            return result;
        }
        for (Object datum : data) {
            result.add(JSONObject.parseObject(JSON.toJSONString(datum)));
        }
        return result;
    }


    /**
     * 转换为数据中心对象
     * @param array 数据中台对象
     * @param clazz 数据中心对象class
     * @param <T>
     * @return
     */
    public static <T> List<T> toAdmMultiEntity(List array, List<String> projection, Class<T> clazz) {
        ArrayNode dataNode = JsonNodeUtils.toArrayNode(array, null, null);
        return toAdmMultiEntity(dataNode, projection, clazz);
    }

    /**
     * 转换为数据中心对象
     * @param array
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> List<T> toAdmMultiEntity(ArrayNode array, List<String> projection, Class<T> clazz) {
        if(ArrayUtil.isEmpty(array)) {
            return null;
        }
        // 转换属性名
        for(int i = 0, j = array.size();i < j;i++) {
            JsonNode node = array.get(i);
            if(node != null && node.isObject()) {
                resetAdmProperty((ObjectNode) node, projection);
            }
        }
        return JsonNodeUtils.toEntity(array, clazz, "infos");
    }

    /**
     * 转换为数据中心对象
     * @param node
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> T toAdmSingleEntity(ObjectNode node, List<String> projection, Class<T> clazz) {
        resetAdmProperty(node, projection);
        return JsonNodeUtils.toEntity(node, clazz, "infos");
    }

    /**
     * 处理dmp转adm对象的属性名
     * @param node
     */
    private static void resetAdmProperty(ObjectNode node, List<String> projection) {
        // 转换属性名 TODO 目前只有创建时间和修改时间需要转换，后续有其他时通过建立映射在此转换即可
        copyProperty(node, "creationTime", "createTime");
        copyProperty(node, "modifiedTime", "lastUpdate");
        copyProperty(node, "objType", "objectType");
        removeProperty(node, projection);
    }

    /**
     * 拷贝属性
     * @param dmpData 数据中台对象
     * @param srcProp 数据中台属性名
     * @param desProp 对应的数据中心属性名
     */
    private static void copyProperty(ObjectNode dmpData, String srcProp, String desProp) {
        if(dmpData == null) {
            return;
        }
        JsonNode value = dmpData.get(srcProp);
        if(value == null || value.isNull()) {
            return;
        }
        dmpData.set(desProp, value);
        dmpData.remove(srcProp);
    }

    /**
     * 删除掉数据中心不需要的属性
     * @param dmpData
     */
    private static void removeProperty(ObjectNode dmpData, List<String> projection){
        if(CollUtil.isNotEmpty(projection)) {
            // 转换为Set，提供遍历效率
            Set<String> fieldSet = CollUtil.newHashSet(projection);
            Iterator<String> fields =  dmpData.fieldNames();
            while (fields.hasNext()) {
                String field = fields.next();
                // 返回结果中包含的不处理
                if(fieldSet.contains(field)) {
                    continue;
                }
                // 去掉不包含的属性
                fields.remove();
            }
        } else {
            // 返回全部时，去掉不需要属性
            List<String> removeList = CollUtil.newArrayList("creator", "groupCode", "valid", "createApp", "virtualCodes", "updateApp", "grouping", "modifier");
            for(String removeField:removeList) {
                dmpData.remove(removeField);
            }
        }
    }

    /**
     * 获取查询条件
     * @param graphCode
     * @param relCode
     * @param id （fromID/targetID）
     * @param direct  0 正向 1反向
     * @return
     */
    public static QueryCriteria transDmpRelationCriteria(ObjectNode objectNode,String graphCode, String relCode, String id, Integer direct){
        QueryCriteria queryCriteria = new QueryCriteria();

        ObjectNode object = JsonNodeFactory.instance.objectNode();
        object.put("graphCode", graphCode);
        object.put("relCode", relCode);
        if(direct == 0){
            //正向查询 objFrom->objTo
            object.put("objFrom", id);
            objectNode.put("relationTo",object);
        }else{
            //反向查询 objTo->objFrom
            object.put("objTo", id);
            objectNode.put("relationFrom",object);
        }
        queryCriteria.setCriteria(objectNode);
        return queryCriteria;
    }

    /**
     * 数据合并
     * @param lists
     * @return
     */
    public static List<JSONObject> combineData(List<JSONObject> lists){
        if(CollUtil.isEmpty(lists)){
            return new ArrayList<>();
        }
        Map<String, List<JSONObject>> map =lists.stream().collect(Collectors.groupingBy(k->k.getString("className"),LinkedHashMap::new,Collectors.toList()));
        //需合并的数据
        List<JSONObject> result = new ArrayList<>();
        for (Map.Entry<String, List<JSONObject>> entry : map.entrySet()) {
            List<JSONObject> value = entry.getValue();

            JSONObject wdDefineTypeRel = new JSONObject();
            JSONArray allObjects = new JSONArray();
            for (JSONObject object : value) {
                wdDefineTypeRel = object;
                JSONArray objects = object.getJSONArray("objects");
                if(objects != null && objects.size()>0){
                    allObjects.addAll(objects);
                }
            }
            wdDefineTypeRel.put("objects",allObjects);
            wdDefineTypeRel.put("count",allObjects == null ? 0 : allObjects.size());

            result.add(wdDefineTypeRel);
        }
        return result;
    }

}
